
/**
 * This behavior will call to Acquia Purge's AJAX path until its purging queue
 * is empty and everything is processed. Also works without on-screen reporting.
 */
Drupal.behaviors.AcquiaPurgeAjax = function() {

  // Declare the trigger path the script will call back home to.
  triggerPath = Drupal.settings.basePath + 'acquia_purge_ajax_processor';

  // Declare reference variables to the main container and its elements.
  apbox = '.acquia_purge_messages';
  apbox_errors = '#aperror';
  apbox_widget = '#apwidget';
  apbox_log = '#aplog';

  // Declare a error counter and a error limit before we stop processing.
  errorCounter = 0;
  errorCounterLimit = 3;

  // Determine if on-screen reporting is enabled or not.
  function uiActivated() {
    if ($(apbox).length > 0) {
      return true;
    }
    return false;
  }

  // Prepare the container element with extra structures that we will need.
  function uiInitialize() {

    // Set a element reference to the apbox container.
    apbox = $(apbox);

    // Wrap the currently existing HTML string into apbox_widget.
    apbox.html("<div id='apwidget'>" + apbox.html() + "</div>");
    apbox_widget = apbox.find(apbox_widget);

    // Prepend the hidden errors container and reference it.
    html = "<div id='aperror' style='display: none;' class='messages ";
    html = html + "error'>&nbsp;</div>";
    apbox.prepend(html);
    apbox_errors = apbox.find(apbox_errors);

    // Append the unordered purge log to the container and reference it.
    html = "<div id='aplog' style='display: none;'><ul></ul></div>";
    apbox.append(html);
    apbox_log = apbox.find(apbox_log);

    // Set padding on the apbox container to have a smooth throbber on Garland.
    apbox.css('padding-left', '2em');

    // As the apbox container is hidden by default, reveal our loading UI.
    apbox.show();
  }

  // Set a visual error in the error container. Returns true when processing
  // could continue and returns false when the error limit has been reached.
  function uiError(message) {
    message = typeof message !== 'undefined' ? message : false;
    message_old = apbox_errors.html();

    // If message wasn't passed we hide and delete the current message.
    if (!message) {
      apbox_errors.slideUp('slow');
      apbox_errors.html('&nbsp;');
      return;
    }
    else {

      // Increase the error counter.
      errorCounter = errorCounter + 1;
    }

    // Don't do anything when the new and old messages are the same.
    if (message == message_old) {
      return (errorCounter < errorCounterLimit) ? true : false;
    }

    // When the box is invisible: set the message and unfold.
    if (!apbox_errors.is(':visible')) {
      apbox_errors.html(message);
      apbox_errors.slideDown('slow');
      return (errorCounter < errorCounterLimit) ? true : false;
    }

    // The only resulting case is a message replacement, handle nicely.
    apn  = "<div class='apn' style='display:none;'>" + message + "</div>";
    apbox_errors.html("<div class='apo'>" + apbox_errors.html() + "</div>");
    apbox_errors.append(apn);
    apbox_errors.find('.apo').slideUp('slow');
    apbox_errors.find('.apn').slideDown('slow', function() {
      apbox_errors.html(message);
    });
    return (errorCounter < errorCounterLimit) ? true : false;
  }

  // Enable the throbber on the widget container.
  function uiThrobberOn() {
    throbber_path = Drupal.settings.basePath + 'misc/throbber.gif';
    throbber = apbox.find('#apthrobr');

    // Create the throbber when it doesn't exist.
    if (!($(throbber).length > 0)) {
      html = "<div id='apthrobr' style='display: none;'>&nbsp;</div>";
      apbox_errors.after(html);
      throbber = apbox.find('#apthrobr');
      throbber.css('background-image', 'url(' + throbber_path + ')');
      throbber.css('background-color', 'transparent');
      throbber.css('background-position', '0px -18px');
      throbber.css('background-repeat', 'no-repeat');
      throbber.css('margin-top', '-21px');
      throbber.css('position', 'relative');
      throbber.css('z-index', '1');
      throbber.css('height', '18px;');
      throbber.css('width', '18px');
      throbber.css('top', '1.8em');
      throbber.css('left', '-1.4em');
    }

    // Then just make it visible.
    throbber.fadeIn(1000);
  }

  // Disable the throbber on the widget container.
  function uiThrobberOff() {
    throbber = apbox.find('#apthrobr');
    throbber.fadeOut(1000);
  }

  // Add new items to the purge log history widget.
  function uiLogHistory(purges) {
    list_items_limit = 10;
    list_items = apbox_log.find('ul');

    // Slowly slide the widget on screen once purges are logged.
    if (!apbox_log.is(':visible')) {
      apbox_log.show();
    }

    // Iterate each URL and append it to the list items.
    $.each(purges, function(key, url) {

      // Check the existing list and add the item if its new to us.
      alreadyInList = false;
      list_items.find('li').each(function(index) {
        if ($(this).text() == url) {alreadyInList = true;}
      });

      // Addition logic when the item is indeed unique.
      if (!alreadyInList) {

        // List quota reached, skip effects.
        if (list_items.find('li').length == list_items_limit) {
          list_items.find('li:first').remove();
          list_items.append("<li>" + url + '</li>');
          list_items.find('li:last').css('list-style', 'none');
        }
        else {
          list_items.append("<li style='display:none;'>" + url + '</li>');
          list_items.find('li:last').css('list-style', 'none');
          list_items.find('li:last').slideDown(1000);
        }
      }
    });
  }

  // Build off the log history viewer and hide its items.
  function uiLogHistoryHide() {
    apbox_log.fadeTo(2000, 0).slideUp(2000);
  }

  // Tear the user interface down and hide it for the user.
  function uiTearDown() {
    uiThrobberOff();
    uiLogHistoryHide();
    uiError();

    // Hide ourselves after 4 seconds.
    setTimeout(function() {apbox.slideUp(1000);}, 4000);
  }

  // Make a request back home and trigger a couple of purges each run.
  function eventLoopRun() {

    // Initialize the throbber.
    if (uiActivated()) {
      uiThrobberOn();
    }

    // Start a recursive call and call ourselves upon success.
    $.ajax({
      url: triggerPath,
      cache: false,
      dataType: "json",
      context: document.body,
      success: function(data) {

        // Replace the inner text with the loaded widget.
        $(apbox_widget).html(data['widget']);

        // Disable the throbber since we're done again.
        if (uiActivated() && (!data['locked'])) {
          uiThrobberOff();
        }

        // Report successfully purged URLs to the GUI's logging widget. @TODO
        if (uiActivated() && (data['purgehistory'].length > 0)) {
          uiLogHistory(data['purgehistory']);
        }

        // Handle error conditions and remove errors when they are gone.
        errorHandlerCallsForHalt = false;
        if (uiActivated() && data['error']) {
          if (!uiError(data['error'])) {
            errorHandlerCallsForHalt = true;
          }
        }
        else if (uiActivated() && (!data['error'])) {
          uiError();
        }

        // Follow up a next request with a 2 seconds pause.
        if (data['running']) {
          if (!errorHandlerCallsForHalt) {
            if (data['locked'] || data['error']) {
              setTimeout(function() {eventLoopRun();}, 10000);
            }
            else {
              setTimeout(function() {eventLoopRun();}, 1000);
            }
          }
          else {
            uiThrobberOff();
          }
        }

        // Start building off the interface since the work is done.
        else {
          if (uiActivated()) {
            uiTearDown();
          }
        }
      },
      error: function(request) {

        // Sometimes requests randomly fail with HTTP 200 (OK), continue
        // processing requests as it most probably just did its work.
        if (Number(request['status']) == 200) {
          if (uiActivated()) {
            uiThrobberOff();
          }
          eventLoopRun();
        }

        // 403 responses indicate that the user does not have access to the
        // AJAX back-end anymore, this happens when someone else initiated
        // a purge sequence or when multiple progress bars fight each-other.
        else if (Number(request['status']) == 403) {
          if (uiActivated()) {
            uiTearDown();
          }
        }

        // Else, report the error occurred and tear the UI partly down.
        else {
          if (uiActivated()) {
            uiThrobberOff();
            uiLogHistoryHide();
            msg = "Something went wrong while communicating with the ";
            msg = msg + "server. Last known response was '";
            msg = msg + request['statusText'] + "' with HTTP code ";
            msg = msg + request['status'] + ".";
            uiError(msg);
          }
        }
      }
    });
  }

  // Initialize the UI when we have detected its base element.
  if (uiActivated()) {
    uiInitialize();
  }

  // Start the cascade of purge events until its marked finished.
  eventLoopRun();

};
